package java.beans;

import java.lang.ref.Reference;
import java.util.ArrayList;
import java.lang.reflect.Method;
import java.lang.ref.WeakReference;
import java.util.List;

public class MethodDescriptor extends FeatureDescriptor
{
    private final MethodRef methodRef;
    private String[] paramNames;
    private List<WeakReference<Class<?>>> params;
    private ParameterDescriptor[] parameterDescriptors;
    
    public MethodDescriptor(Method method) {
        this(method, (ParameterDescriptor[])null);
    }
    
    public MethodDescriptor(Method method, ParameterDescriptor[] array) {
        this.methodRef = new MethodRef();
        this.setName(method.getName());
        this.setMethod(method);
        this.parameterDescriptors = ((array != null) ? ((ParameterDescriptor[])array.clone()) : null);
    }
    
    public synchronized Method getMethod() {
        Method method = this.methodRef.get();
        if (method == null) {
            Class<Object> class0 = this.getClass0();
            String name = this.getName();
            if (class0 != null && name != null) {
                Class[] params = this.getParams();
                if (params == null) {
                    int i = 0;
                    while (i < 3) {
                        method = Introspector.findMethod(class0, name, i, (Class[])null);
                        if (method != null) {
                            break;
                        }
                        ++i;
                    }
                }
                else {
                    method = Introspector.findMethod(class0, name, params.length, params);
                }
                this.setMethod(method);
            }
        }
        return method;
    }
    
    private synchronized void setMethod(Method method) {
        if (method == null) {
            return;
        }
        if (this.getClass0() == null) {
            this.setClass0(method.getDeclaringClass());
        }
        this.setParams(FeatureDescriptor.getParameterTypes(this.getClass0(), method));
        this.methodRef.set(method);
        return;
    }
    
    private synchronized void setParams(Class<?>[] array) {
        if (array == null) {
            return;
        }
        this.paramNames = new String[array.length];
        this.params = new ArrayList(array.length);
        for (int i = 0; i < array.length; ++i) {
            this.paramNames[i] = array[i].getName();
            this.params.add(new WeakReference(array[i]));
        }
        return;
    }
    
    String[] getParamNames() {
        return this.paramNames;
    }
    
    private synchronized Class<?>[] getParams() {
        Class[] array = new Class[this.params.size()];
        int i = 0;
        while (i < this.params.size()) {
            Class<Object> clazz = (Class)((Reference)this.params.get(i)).get();
            if (clazz == null) {
                return null;
            }
            array[i] = clazz;
            ++i;
        }
        return array;
    }
    
    public ParameterDescriptor[] getParameterDescriptors() {
        return (this.parameterDescriptors != null) ? ((ParameterDescriptor[])this.parameterDescriptors.clone()) : null;
    }
    
    private static Method resolve(Method method, Method method2) {
        if (method == null) {
            return method2;
        }
        if (method2 == null) {
            return method;
        }
        return (!method.isSynthetic() && method2.isSynthetic()) ? method : method2;
    }
    
    MethodDescriptor(MethodDescriptor methodDescriptor, MethodDescriptor methodDescriptor2) {
        super(methodDescriptor, methodDescriptor2);
        (this.methodRef = new MethodRef()).set(resolve(methodDescriptor.methodRef.get(), methodDescriptor2.methodRef.get()));
        this.params = methodDescriptor.params;
        if (methodDescriptor2.params != null) {
            this.params = methodDescriptor2.params;
        }
        this.paramNames = methodDescriptor.paramNames;
        if (methodDescriptor2.paramNames != null) {
            this.paramNames = methodDescriptor2.paramNames;
        }
        this.parameterDescriptors = methodDescriptor.parameterDescriptors;
        if (methodDescriptor2.parameterDescriptors != null) {
            this.parameterDescriptors = methodDescriptor2.parameterDescriptors;
        }
    }
    
    MethodDescriptor(MethodDescriptor methodDescriptor) {
        super(methodDescriptor);
        (this.methodRef = new MethodRef()).set(methodDescriptor.getMethod());
        this.params = methodDescriptor.params;
        this.paramNames = methodDescriptor.paramNames;
        if (methodDescriptor.parameterDescriptors != null) {
            int length = methodDescriptor.parameterDescriptors.length;
            this.parameterDescriptors = new ParameterDescriptor[length];
            for (int i = 0; i < length; ++i) {
                this.parameterDescriptors[i] = new ParameterDescriptor(methodDescriptor.parameterDescriptors[i]);
            }
        }
    }
    
    @Override
    void appendTo(StringBuilder sb) {
        FeatureDescriptor.appendTo(sb, "method", this.methodRef.get());
        if (this.parameterDescriptors != null) {
            sb.append("; parameterDescriptors={");
            ParameterDescriptor[] parameterDescriptors = this.parameterDescriptors;
            for (int length = parameterDescriptors.length, i = 0; i < length; ++i) {
                sb.append(parameterDescriptors[i]).append(", ");
            }
            sb.setLength(sb.length() - 2);
            sb.append("}");
        }
    }
}
